<?php
/**
 * @package     Joomla.Site
 * @subpackage  Templates.protostar
 *
 * @copyright   Copyright (C) 2005 - 2015 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

/**
 * This is a file to add template specific chrome to module rendering.  To use it you would
 * set the style attribute for the given module(s) include in your template to use the style
 * for each given modChrome function.
 *
 * eg. To render a module mod_test in the submenu style, you would use the following include:
 * <jdoc:include type="module" name="test" style="submenu" />
 *
 * This gives template designers ultimate control over how modules are rendered.
 *
 * NOTICE: All chrome wrapping methods should be named: modChrome_{STYLE} and take the same
 * two arguments.
 */

/*
 * Module chrome for rendering the module in a submenu
 */


function modChrome_no($module, &$params, &$attribs)
{
	if ($module->content)
	{
		echo $module->content;
	}
}

function modChrome_well($module, &$params, &$attribs)
{
	$moduleTag     = $params->get('module_tag', 'div');
	$bootstrapSize = (int) $params->get('bootstrap_size', 0);
	$moduleClass   = $bootstrapSize != 0 ? ' span' . $bootstrapSize : '';
	$headerTag     = htmlspecialchars($params->get('header_tag', 'h3'));
	$headerClass   = htmlspecialchars($params->get('header_class', 'page-header'));

	if ($module->content)
	{
		echo '<' . $moduleTag . ' class="well ' . htmlspecialchars($params->get('moduleclass_sfx')) . $moduleClass . '">';

			if ($module->showtitle)
			{
				echo '<' . $headerTag . ' class="' . $headerClass . '">' . $module->title . '</' . $headerTag . '>';
			}

			echo $module->content;
		echo '</' . $moduleTag . '>';
	}
}

function modChrome_luxury_black_header($module, &$params, &$attribs)
{
	$moduleTag     = $params->get('module_tag', 'div');
	$bootstrapSize = (int) $params->get('bootstrap_size', 0);
	$moduleClass   = $bootstrapSize != 0 ? ' span' . $bootstrapSize : '';
	$headerTag     = htmlspecialchars($params->get('header_tag', 'h3'));
	$headerClass   = htmlspecialchars($params->get('header_class', 'black_header'));
	
	$app  = JFactory::getApplication();
	$template = $app->getTemplate(true);
	$temp_name = $template->template;

	if ($module->content)
	{
      
		echo '<' . $moduleTag . ' class="moduletable ' . htmlspecialchars($params->get('moduleclass_sfx')) . $moduleClass . '">';

			if ($module->showtitle)
			{
				echo '<div class="section_title text-center">'; 
				echo '<' . $headerTag . ' class="' . $headerClass . '">' . $module->title . '</' . $headerTag . '>';
				echo '<img src="templates/'.$temp_name.'/img/gray-star.png" class="gray_star" alt="">';
				echo '</div>';
			}

			echo $module->content;
		echo '</' . $moduleTag . '>';
	}
}

function modChrome_luxury_white_header($module, &$params, &$attribs)
{
	$moduleTag     = $params->get('module_tag', 'div');
	$bootstrapSize = (int) $params->get('bootstrap_size', 0);
	$moduleClass   = $bootstrapSize != 0 ? ' span' . $bootstrapSize : '';
	$headerTag     = htmlspecialchars($params->get('header_tag', 'h3'));
	$headerClass   = htmlspecialchars($params->get('header_class', 'black_header white_text'));
	
	$app  = JFactory::getApplication();
	$template = $app->getTemplate(true);
	$temp_name = $template->template;

	if ($module->content)
	{
      
		echo '<' . $moduleTag . ' class="moduletable ' . htmlspecialchars($params->get('moduleclass_sfx')) . $moduleClass . '">';

			if ($module->showtitle)
			{
				echo '<div class="section_title text-center">'; 
				echo '<' . $headerTag . ' class="' . $headerClass . '">' . $module->title . '</' . $headerTag . '>';
				echo '<img src="templates/'.$temp_name.'/img/white-star.png" class="white_star" alt="">';
				echo '</div>';
			}

			echo $module->content;
		echo '</' . $moduleTag . '>';
	}
}
function modChrome_luxury_sidebar_header($module, &$params, &$attribs)
{
	$moduleTag     = $params->get('module_tag', 'div');
	$bootstrapSize = (int) $params->get('bootstrap_size', 0);
	$moduleClass   = $bootstrapSize != 0 ? ' span' . $bootstrapSize : '';
	$headerTag     = htmlspecialchars($params->get('header_tag', 'h3'));
	$headerClass   = htmlspecialchars($params->get('header_class', 'sidebar_title'));

	if ($module->content)
	{
      
		echo '<' . $moduleTag . ' class="moduletable ' . htmlspecialchars($params->get('moduleclass_sfx')) . $moduleClass . '">';

			if ($module->showtitle)
			{
				 
				echo '<' . $headerTag . ' class="' . $headerClass . '">' . $module->title . '</' . $headerTag . '>';
				echo '<div class="sidebar_header_border"></div>';
				
			}

			echo $module->content;
		echo '</' . $moduleTag . '>';
	}
}

function modChrome_luxury_vm_prd($module, &$params, &$attribs)
{
	$moduleTag     = $params->get('module_tag', 'div');
	$bootstrapSize = (int) $params->get('bootstrap_size', 0);
	$moduleClass   = $bootstrapSize != 0 ? ' span' . $bootstrapSize : '';
	$headerTag     = htmlspecialchars($params->get('header_tag', 'h3'));
	$headerClass   = htmlspecialchars($params->get('header_class', 'sidebar_title'));

	if ($module->content)
	{
      
		echo '<' . $moduleTag . ' class="moduletable ' . htmlspecialchars($params->get('moduleclass_sfx')) . $moduleClass . '">';
        echo '<div class="bgcolor">';
			if ($module->showtitle)
			{
				 
				echo '<' . $headerTag . ' class="' . $headerClass . '">' . $module->title . '</' . $headerTag . '>';
				echo '<div class="sidebar_header_border"></div>';
				
			}

			echo $module->content;
		 echo '</div>';	
		echo '</' . $moduleTag . '>';
	}
}
function modChrome_luxury_topmenu($module, &$params, &$attribs)
{
	$moduleTag     = $params->get('module_tag', 'div');
	$bootstrapSize = (int) $params->get('bootstrap_size', 0);
	$moduleClass   = $bootstrapSize != 0 ? ' span' . $bootstrapSize : '';
	//$headerTag     = htmlspecialchars($params->get('header_tag', 'h3'));
	$headerClass   = htmlspecialchars($params->get('header_class', 'topmenu_header'));

	if ($module->content)
	{
		echo '<' . $moduleTag . ' class="moduletable ' . htmlspecialchars($params->get('moduleclass_sfx')) . $moduleClass . '">';
           
			if ($module->showtitle)
			{ ?>
				<a title="<?php echo $module->title;?>" class="dropdown-toggle" data-toggle="dropdown">
				  <span class="user-text text-uppercase"><?php echo $module->title;?></span>&nbsp;
				  <i class="fa fa-angle-down"></i>
				</a> 
				
			<?php  }

			echo $module->content;
		echo '</' . $moduleTag . '>';
	}
}